from __future__ import annotations
from crypticorn.klines import (
    ApiClient,
    Configuration,
    FundingRatesApi,
    HealthCheckApi,
    OHLCVDataApi,
    SymbolsApi,
    UDFApi,
)
from crypticorn.common import optional_import


class FundingRatesApiWrapper(FundingRatesApi):
    """
    A wrapper for the FundingRatesApi class.
    """
    async def get_funding_rates_fmt(self, *args, **kwargs) -> pd.DataFrame:  # type: ignore
        """
        Get the funding rates in a pandas DataFrame.
        """
        pd = optional_import("pandas", "extra")
        response = await self.get_funding_rates_without_preload_content(*args, **kwargs)
        response.raise_for_status()
        json_data = await response.json()
        return pd.DataFrame(json_data)


class OHLCVDataApiWrapper(OHLCVDataApi):
    """
    A wrapper for the OHLCVDataApi class.
    """

    async def get_ohlcv_data_fmt(self, *args, **kwargs) -> pd.DataFrame:  # type: ignore
        """
        Get the OHLCV data in a pandas DataFrame.
        """
        pd = optional_import("pandas", "extra")
        response = await self.get_ohlcv_without_preload_content(*args, **kwargs)
        response.raise_for_status()
        json_data = await response.json()
        return pd.DataFrame({
                    "timestamp": json_data["timestamp"],
                    "open": json_data["open"],
                    "high": json_data["high"],
                    "low": json_data["low"],
                    "close": json_data["close"],
                    "volume": json_data["volume"]
                })


class SymbolsApiWrapper(SymbolsApi):
    """
    A wrapper for the SymbolsApi class.
    """

    async def get_symbols_fmt(self, *args, **kwargs) -> pd.DataFrame:  # type: ignore
        """
        Get the symbols in a pandas DataFrame.
        """
        pd = optional_import("pandas", "extra")
        response = await self.get_klines_symbols_without_preload_content(*args, **kwargs)
        response.raise_for_status()
        json_data = await response.json()
        return pd.DataFrame(json_data["data"], columns=["symbol"])


class KlinesClient:
    """
    A client for interacting with the Crypticorn Klines API.
    """

    def __init__(
        self,
        config: Configuration,
    ):
        self.config = config
        self.base_client = ApiClient(configuration=self.config)
        # Instantiate all the endpoint clients
        self.funding = FundingRatesApiWrapper(self.base_client)
        self.ohlcv = OHLCVDataApiWrapper(self.base_client)
        self.symbols = SymbolsApiWrapper(self.base_client)
        self.udf = UDFApi(self.base_client)
        self.health = HealthCheckApi(self.base_client)
